% Make predictions for each model using Boosted MaxEnt model
%
% regressors : T-by-(K-1) cell array containing regressors that can handle
%            d-dimensional data.  T = # of iters, K = # of labels
% steps  : T-by-1 array containing step-sizes for each iteration
% regressor_type
% X      : M-by-D matrix containing M feature samples of dimension d
% t      : (optional) number of iterations of the model to use
%             If t=0, then P returns uniform probabilities
%
% P      : M-by-K matrix probability matrix for assigning each sample to
%            each class
function P = bme_predict(regressors, steps, regressor_type, X, nbr_labels, t)
assert(nargin == 5 || nargin == 6)

nbr_samples = size(X, 1);
nbr_regressors = size(regressors, 1);

assert(sum(steps < 0) == 0);
assert(nbr_regressors == length(steps));
if (nargin == 6)
    assert(0 <= t && t <= nbr_regressors);
    nbr_regressors = t;
end

% Compute exp(score) for assigning each label to each sample    
S = regressor_predict(regressors, steps, regressor_type, X, nbr_regressors, nbr_labels);
S = exp(S);

% Compute normalizer score for each sample (sum_k exp(w_k .* x))
Z = sum(S, 2);

% Compute N-by-K matrix probability matrix for each label assignment
Z = repmat(Z, 1, nbr_labels);
P = S ./ Z;

% Verify P sums to 1 for each entry
assert( sum( abs(ones(nbr_samples,1) - sum(P,2) ) ) < 1e-6);
end
