% regressors : T-by-(K-1) cell array containing regressors that can handle
%            d-dimensional data.  T = # of iters, K = # of labels
% steps  : T-by-1 array containing step-sizes for each iteration
% regressor_type : 
% X      : M-by-D matrix containing M feature samples of dimension d
% t      : number of regressors to use (t <= T)
%
% S  : M-by-(K-1) each M samples' exp(score) matrix for each label assignment, 
%        except the last one
function S = regressor_predict(regressors, steps, regressor_type, X, t, nbr_labels)
assert(nargin == 6)

nbr_samples = size(X, 1);
S = zeros(nbr_samples, nbr_labels);  % Score matrix: w_k .* x_i

for regressor_idx = 1:t
    curr_regressor = regressors{regressor_idx};
    
    % 1 regressor per label
    for label_idx=1:(nbr_labels)
        label_regressor = curr_regressor{label_idx};
        
        curr_outputs = zeros(nbr_samples, 1);
        if (strcmp(regressor_type, 'RandomForest'))
            curr_outputs(:, 1) = regRF_predict(X, label_regressor);
        else
            error(['Not implemented: ', regressor_type]);
        end
        
        S(:,label_idx) = S(:,label_idx) + (steps(regressor_idx) .* curr_outputs);
    end
end
