% Trains a regressor
%
function [R] = regressor_train(regressor_type, X, y_vec, min_target)
assert(nargin == 4);
assert(size(X,1) == size(y_vec,1));

nbr_labels = size(y_vec, 2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1 regressor per-label
X_valid = cell(nbr_labels, 1);
y_valid = cell(nbr_labels, 1);
% Determine sample indices that have sufficient target value
% (i.e. not epsilon close to zero)
for label_idx=1:nbr_labels
    valid_indices = abs(y_vec(:, label_idx)) > min_target;
    X_valid{label_idx} = X(valid_indices, :);
    y_valid{label_idx} = y_vec(valid_indices, label_idx);
end

R = cell(1, nbr_labels);
for label_idx=1:nbr_labels
    curr_X_valid = X_valid{label_idx};
    curr_y_valid = y_valid{label_idx};
    
    if (strcmp(regressor_type, 'RandomForest')) % Random forest
        extra_options.replace = 1;
        extra_options.nodesize = 15;
        NBR_TREES = 10;
        NBR_SAMPLE_DIMS = floor(sqrt(size(curr_X_valid,2)));
        R{label_idx} = regRF_train(curr_X_valid, curr_y_valid, NBR_TREES, NBR_SAMPLE_DIMS, extra_options);
    else
        error(['Not implemented: ', regressor_type]);
    end
    
    clear curr_X_valid;
    clear curr_y_valid;
end
