% Creates K label confidence maps for each label
%
% seg_map    : M-by-N segment image [0,S-1]
% seg_labels : S-by-K label predictions for each segment
%
% ilp        : 1-by-K histogram computed by weighted averaging predictions
function ilp = create_image_level_prior(seg_map, seg_labels)
nbr_segs = max(max(seg_map)) + 1;
assert(min(min(seg_map)) == 0);
assert(nbr_segs == size(seg_labels,1));
total_area = size(seg_map,1) * size(seg_map,2);

nbr_labels = size(seg_labels,2);
ilp = zeros(1,nbr_labels);

for i=1:nbr_segs
    % scale histogram by percent of segment wrt to image
    curr_area = sum(sum(seg_map == (i-1)));
    curr_ratio = curr_area / total_area;
    curr_histo = seg_labels(i,:) .* curr_ratio;
    ilp = ilp + curr_histo;
end

% Normalize
assert(sum(ilp) > 1e-5);
ilp = ilp ./ sum(ilp);
