% Creates K label confidence maps for each label
%
% seg_map    : M-by-N segment image [0,S-1]
% seg_labels : S-by-K label predictions for each segment
%
% conf_maps  : K-by-1 cell vector containing M-by-N matrices w/ [0,1] confidences
function conf_maps = create_label_conf_maps(seg_map, seg_labels)
nbr_segs = size(seg_labels,1);
assert(nbr_segs == max(max(seg_map))+1);
nbr_labels = size(seg_labels,2);

% For each label, create an image with the confidence per-pixel
conf_maps = cell(nbr_labels,1);
for k=1:nbr_labels
    conf_maps{k} = zeros(size(seg_map));
    for s=1:nbr_segs
        conf_maps{k}(seg_map == (s-1)) = seg_labels(s,k);
    end
end
