% path_seg_tree_root : Root dir of segmentation tree for train & test features and labels
% path_machine_root  : Root dir of machine/classifier
% path_inf_tree_root : Root dir of inference tree where can find parent
%                       segment's predicted label histogram (same as 
%                       path_machine_root during training)
%
function infer_level(path_seg_tree_root, path_machine_root, path_inf_tree_root, ...
                     curr_scale, parent_scale, feature_names, img_basenames, use_bme)
assert(nargin == 8);

path_inf_segment_labels = [path_inf_tree_root, '/scale_', num2str(curr_scale), '/inf_segment_labels'];
path_inf_conf_maps = [path_inf_tree_root, '/scale_', num2str(curr_scale), '/inf_conf_maps'];
mkdir(path_inf_segment_labels);
mkdir(path_inf_conf_maps);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load normalization, stage1 and stage 2 models
normalize_dat = load([path_machine_root, '/scale_', num2str(curr_scale), '/normalize.mat']);
means = normalize_dat.means;
stds = normalize_dat.stds;
%%%%%%%%%
% Load stage 1 model
flr_model_stage1 = load([path_machine_root, '/scale_', num2str(curr_scale), '/flr_model_stage1.mat']);
flr_model_stage1 = flr_model_stage1.flr_model_stage1;
if (use_bme)
    bme1_regressors  = flr_model_stage1.regressors;
    bme1_steps       = flr_model_stage1.steps;
    bme1_regressor_t = flr_model_stage1.regressor_type;
    nbr_labels       = flr_model_stage1.nbr_labels;
    eme1_w = [];
else
    bme1_regressors = []; bme1_steps = []; bme1_regressor_t = []; nbr_labels = Inf;
    eme1_w = flr_model_stage1;
end
clear flr_model_stage1;
%%%%%%%%%
% Load stage 2 model
if (parent_scale >= 0)
    flr_model_stage2 = load([path_machine_root, '/scale_', num2str(curr_scale), '/flr_model_stage2.mat']);
    flr_model_stage2 = flr_model_stage2.flr_model_stage2;
    if (use_bme)
        bme2_regressors  = flr_model_stage2.regressors;
        bme2_steps       = flr_model_stage2.steps;
        bme2_regressor_t = flr_model_stage2.regressor_type;
        assert(nbr_labels == flr_model_stage2.nbr_labels);
        eme2_w = [];
    else
        bme2_regressors = []; bme2_steps = []; bme2_regressor_t = [];
        eme2_w = flr_model_stage2;
    end
    clear flr_model_stage2;
else
    % Needed for parfor (for some reason)
    bme2_regressors = []; bme2_steps = []; bme2_regressor_t = [];
    eme2_w = [];
end

nbr_imgs = length(img_basenames);
parfor i=1:nbr_imgs
    i
    curr_basename = char(img_basenames(i));
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % Load features
    [S_x, S_y_par_inf, S_y_gt, parent_ilp, S_lcontext, seg_map] = ... 
        load_img_segs2(path_seg_tree_root, path_inf_tree_root, curr_basename, feature_names, curr_scale, parent_scale);   
    nbr_segs = size(S_x,1);
    assert(nbr_segs == size(S_y_gt, 1));
    if (~isempty(parent_ilp))
        assert(size(parent_ilp,1) == 1);
    end
    parent_ilp = repmat(parent_ilp, nbr_segs, 1);
    bias = ones(nbr_segs,1);
    
    S_x = normalize_data(S_x, means, stds);
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate Stage 1 Predictions
    X1 = [S_x, S_y_par_inf, parent_ilp, S_lcontext, bias];
    if (use_bme)
        y1 = bme_predict(bme1_regressors, bme1_steps, bme1_regressor_t, X1, nbr_labels);
    else
        y1 = eme_predict(eme1_w, X1);
    end
    X1 = [];
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate Stage 2 Predictions
    % Stop if top level
    if (parent_scale < 0)
        y1_label_conf_maps = create_label_conf_maps(seg_map, y1);
        save_predictions2(curr_basename, y1, path_inf_segment_labels, y1_label_conf_maps, path_inf_conf_maps);
    else
        % Create label context features around each segment
        y1_label_conf_maps = create_label_conf_maps(seg_map, y1);
        y1_label_context = cell2mat(create_label_context_feature(y1_label_conf_maps, seg_map));
        X2 = [S_x, S_y_par_inf, parent_ilp, y1, y1_label_context, bias];
        if (use_bme)
            y2 = bme_predict(bme2_regressors, bme2_steps, bme2_regressor_t, X2, nbr_labels);
        else
            y2 = eme_predict(eme2_w, X2);
        end
        X2 = [];

        y2_label_conf_maps = create_label_conf_maps(seg_map, y2);
        save_predictions2(curr_basename, y2, path_inf_segment_labels, y2_label_conf_maps, path_inf_conf_maps);
    end    
end
